# fix conflict between libopenmpi-devel and libmpi-devel
# both provides devel(libmpi(64bit)) so let's only openmpi keeping it
%global __provides_exclude %{?__provides_exclude:%__provides_exclude|}devel\\(libmpi\\)
%global __provides_exclude %{?__provides_exclude:%__provides_exclude|}devel\\(libmpi\\(64bit\\)\\)
%global __requires_exclude %{?__requires_exclude:%__requires_exclude|}devel\\(libmpi.*

%define	major		12
%define	libname		%mklibname mpi %{major}
%define devname 	%mklibname mpi -d
%define mpihome		/home/mpi

%bcond_without legacy
%bcond_with slurm

Name: 		mpich
Version:	4.3.0
Release: 	%mkrel 0.1
License:	MIT
Group:          System/Cluster
Summary: 	A high-performance implementation of MPI
URL:		https://www.mpich.org
Source0: 	https://www.mpich.org/static/downloads/%{version}/%{name}-%{version}.tar.gz
Source1:	mpich.macros
Source3:	mpich.pth.py3
Patch0:		mpich-modules.patch
Patch1001:	0001-Drop-real128.patch
# Fedora:
Patch10:	mpich-configure-max_align_t.patch
Patch11:	mpich-json-configure-__thread.patch
# FreeBSD:
Patch12:	mpich-4.3.0-test-mpi_configure.ac.patch
Patch13:	mpich-4.3.0-test-mpi_configure.patch
# OpenSuse:
Patch14:	mpich-autogen-setup-json-c.patch

BuildRequires:	gcc-gfortran
BuildRequires:	hwloc-devel >= 2.0
BuildRequires:	json-c-devel
BuildRequires:	libfabric-devel
%{?with_slurm:
BuildRequires:	slurm-devel }
# For python3_sitearch
BuildRequires:	python3-devel
Provides:	mpich2 = %{version}-%{release}
Obsoletes:	mpich2 < 3.0

%description
MPICH is a high-performance and widely portable implementation of the Message
Passing Interface (MPI) standard (MPI-1, MPI-2 and MPI-3). The goals of MPICH
are: (1) to provide an MPI implementation that efficiently supports different
computation and communication platforms including commodity clusters (desktop
systems, shared-memory systems, multicore architectures), high-speed networks
(10 Gigabit Ethernet, InfiniBand, Myrinet, Quadrics) and proprietary high-end
computing systems (Blue Gene, Cray) and (2) to enable cutting-edge research in
MPI through an easy-to-extend modular framework for other derived
implementations.

The mpich binaries in this RPM packages were configured to use the default
process manager (Hydra) using the default device (ch3). The ch3 device
was configured with support for the nemesis channel that allows for
shared-memory and TCP/IP sockets based communication.

%package -n %{devname}
Summary:        Development files for mpich
Group:          System/Cluster
Requires:	%{libname} = %{version}-%{release}
Provides:       lib%{name}-devel = %{version}-%{release}
Provides:	%{name}-devel = %{version}-%{release}
Provides:	mpich2-devel = %{version}-%{release}
Obsoletes:	%{_lib}mpich2-devel < 3.0

%description -n %{devname}
Contains development headers and libraries for mpich.

%package doc
Summary:        Documentations and examples for mpich
Group:          Documentation
Obsoletes:      mpich2-doc < 3.0

%description doc
Contains documentations, examples and man-pages for mpich.

%package -n python3-mpich
Summary:        mpich support for Python 3
Group:          Development/Python

%description -n python3-mpich
mpich support for Python 3.

%package -n %{libname}
Summary:	Shared Libraries for MPICH
Group:		System/Cluster
Provides:	lib%{name} = %{version}-%{release}
Conflicts:	%{_lib}openmpi12 < 2.0.1
Obsoletes:	%{_lib}mpich2_1.2 < 1.2.1-11

%description  -n %{libname}
Shared Libraries for MPICH.

%package -n mpicc
Summary:	The MPICH wrapper over the C compiler
Group:		Development/C
Requires:	gcc >= 3.2
Requires:	%{devname} = %{version}-%{release}
Obsoletes:	mpi2cc < 3.0

%description -n mpicc
MPICH is a freely available, portable implementation of MPI, the Standard
for message-passing libraries.

This package provides the shell script mpicc, with headers, which allows to
compile C programs using the MPICH libraries.

%package -n mpicxx
Summary:	The MPICH wrapper over the C++ compiler
Group:		Development/C++
Requires:	gcc-c++ >= 3.2
Requires:	%{devname} = %{version}-%{release}
Obsoletes:	mpi2cxx < 3.0

%description -n mpicxx
MPICH is a freely available, portable implementation of MPI, the Standard
for message-passing libraries.

This package provides the shell script mpiCC, with headers, which allows to
compile C++ programs using the MPICH libraries.

%package -n mpif90
Summary:	The MPICH wrapper over the Fortran 90 compiler
Group:		System/Cluster
Requires:	gcc-gfortran >= 3.2
Requires:	%{devname} = %{version}-%{release}
Obsoletes:	mpi2f90 < 3.0
Obsoletes:	mpi2f77 < 3.0

%description -n mpif90
MPICH is a freely available, portable implementation of MPI, the Standard
for message-passing libraries.

This package provides the shell script mpif90, with headers, which allows to
compile Fortran 90 (NOT Fortran 77!) programs using the MPICH libraries.

# We only compile with gcc, but other people may want other compilers.
# Set the compiler here.
%{!?opt_cc: %global opt_cc gcc}
%{!?opt_fc: %global opt_fc gfortran}
%{!?opt_f77: %global opt_f77 gfortran}
# Optional CFLAGS to use with the specific compiler...gcc doesn't need any,
# so uncomment and undefine to NOT use
%{!?opt_cc_flags: %global opt_cc_flags %{build_cflags}}
%{!?opt_cxx_flags: %global opt_cxx_flags %{build_cxxflags}}
%{!?opt_fc_flags: %global opt_fc_flags %{build_fflags}}
%{!?opt_f77_flags: %global opt_f77_flags %{build_fflags}}

%ifarch s390
%global m_option -m31
%endif

%ifarch %{ix86} x86_64
%global m_option -m%{__isa_bits}
%endif

%ifarch %{arm} aarch64 %{mips}
%global m_option ""
%endif

%if %{with legacy}
# Legacy mpich setup
%global selected_channels ch3:nemesis
%else
# Default
%global selected_channels ch4:ofi
%endif
# UCX - not available for Mageia yet
#%%global selected_channels ch4:ucx

%ifarch %{ix86} x86_64 s390 %{arm} aarch64
%global XFLAGS -fPIC
%endif

%prep
%setup -q
%autopatch -p1 -M 100
%ifarch %{arm}
%patch -p1 -P 1001
%endif

# Dissable romio tests
sed -e "s/ \. test/ \./" -i src/mpi/romio/Makefile.{am,in}

%build
# Set variables/flags(defaults to Mageia's generic build flags)
export FC=%{opt_fc}
export F77=%{opt_f77}
export CFLAGS="%build_cflags %{m_option} %{?XFLAGS}"
export CXXFLAGS="%build_cxxflags %{m_option} %{?XFLAGS}"
export FCFLAGS="%build_fflags %{m_option} %{?XFLAGS}"
export FFLAGS="%build_fflags %{m_option} %{?XFLAGS} -fallow-argument-mismatch"
export LDFLAGS="%{ldflags} -lm -Wl,-z,noexecstack"
export MPICHLIB_CFLAGS="%{?opt_cc_flags}"
export MPICHLIB_CXXFLAGS="%{opt_cxx_flags}"
export MPICHLIB_FCFLAGS="%{?opt_fc_flags}"
export MPICHLIB_FFLAGS="%{?opt_f77_flags}"

./autogen.sh --without-ucx --without-ofi --without-json
%configure								\
	--enable-shared						\
	--enable-static=no					\
	--disable-rpath						\
	--disable-silent-rules				\
	--with-device=%{selected_channels}	\
	--with-pm=hydra:gforker				\
	%{?with_slurm:--with-slurm }		\
	--with-hwloc						\
	--with-libfabric					\
	--with-json							\
	--includedir=%{_includedir}/%{name}-%{_arch}	\
	--datadir=%{_datadir}/%{name}					\
	--docdir=%{_datadir}/%{name}/doc				\
	--htmldir=%{_datadir}/%{name}/doc/html

# work-around libtool error: cannot determine absolute directory name of 'system/lib'
mkdir -p system/lib

%make_build

%install
%make_install

# Install the RPM macros
install -pDm0644 %{SOURCE1} %{buildroot}%{_rpmconfigdir}/macros.d/macros.%{name}

cat > README.urpmi <<EOF
Post-installation procedure:
- create a user with constant uid/gid and shared home directory on all nodes
- ensure this user has rsh/ssh access to all nodes
For instance, a solution could be:
	# groupadd -g 12384 -r -f mpi
	# useradd -u 12384 -g mpi -d %{mpihome} -r -s /bin/bash mpi -p "" -m
	# mkpasswd -l 32 | passwd --stdin mpi 2>&1 > /dev/null
	# ssh-keygen -f %{mpihome}/.ssh/id_dsa -t dsa -N "" 2>&1 > /dev/null
	# chown mpi:mpi %{mpihome}/.ssh/id_dsa
	# chmod 600 %{mpihome}/.ssh/id_dsa
EOF

# Install the .pth files
mkdir -p %{buildroot}%{python3_sitearch}/%{name}
install -pDm0644 %{SOURCE3} %{buildroot}%{python3_sitearch}/%{name}.pth

rm -fr %{buildroot}%{_libdir}/pkgconfig/yaksa.pc

find %{buildroot} -type f -name "*.la" -delete

%check
%ifnarch %{arm}
%make_build check || :
%endif

%files
%doc COPYRIGHT README.urpmi
%{_bindir}/hydra*
%{_bindir}/mpichversion
%{_bindir}/mpiexec*
%{_bindir}/mpirun
%{_bindir}/mpivars
%{_bindir}/parkill
%{_mandir}/man1/hydra_*.1*
%{_mandir}/man1/mpiexec.1*

%files -n %{libname}
%{_libdir}/*.so.%{major}{,.*}

%files -n %{devname}
%doc COPYRIGHT
%{_includedir}/%{name}-%{_arch}/
%{_libdir}/*.so
%{_libdir}/pkgconfig/*.pc
%{_rpmconfigdir}/macros.d/macros.%{name}
%{_mandir}/man3/*.3*

%files doc
%dir %{_datadir}/%{name}
%{_datadir}/%{name}/doc/

%files -n mpicc
%doc COPYRIGHT
%{_bindir}/mpicc
%{_mandir}/man1/mpicc.1*

%files -n mpicxx
%doc COPYRIGHT
%{_bindir}/mpicxx
%{_bindir}/mpic++
%{_mandir}/man1/mpicxx.1*

%files -n mpif90
%doc COPYRIGHT
%{_bindir}/mpifort
%{_bindir}/mpif90
%{_bindir}/mpif77
%{_mandir}/man1/mpifort.1*

%files -n python3-mpich
%dir %{python3_sitearch}/%{name}
%{python3_sitearch}/%{name}.pth


%changelog
* Tue Jun 17 2025 aurr <aurr> 4.3.0-0.1.mga10
+ Revision: 2192358
- new version: 4.3.0

* Tue Mar 04 2025 wally <wally> 3.4.1-6.mga10
+ Revision: 2160396
- rebuild for python3.13

* Thu Oct 17 2024 daviddavid <daviddavid> 3.4.1-5.mga10
+ Revision: 2107719
- disable failing test_double_serializer

* Mon May 22 2023 daviddavid <daviddavid> 3.4.1-4.mga9
+ Revision: 1957678
- fix conflict between libopenmpi-devel and libmpi-devel
  both provides "devel(libmpi(64bit))" so let's only openmpi keeping it
- remove unneeded yaksa.pc pkgconfig file

* Fri Apr 01 2022 umeabot <umeabot> 3.4.1-2.mga9
+ Revision: 1838141
- Mageia 9 Mass Rebuild

* Sun Jan 30 2022 mrambo3501 <mrambo3501> 3.4.1-1.mga9
+ Revision: 1770613
- version 3.4.1
- add upstream patch which fixes a json test failure

* Thu Aug 20 2020 joequant <joequant> 3.3.2-3.mga8
+ Revision: 1616699
- fix build for gfortran 10.2

* Fri Feb 21 2020 umeabot <umeabot> 3.3.2-2.mga8
+ Revision: 1547786
- Mageia 8 Mass Rebuild

* Mon Feb 10 2020 daviddavid <daviddavid> 3.3.2-1.mga8
+ Revision: 1488354
- new version: 3.3.2
+ wally <wally>
- replace deprecated %%configure2_5x

* Fri Oct 18 2019 daviddavid <daviddavid> 3.3.1-4.mga8
+ Revision: 1454463
- disable testsuite on arm for now

* Fri Oct 18 2019 daviddavid <daviddavid> 3.3.1-3.mga8
+ Revision: 1454433
- add workaround to fix build on arm

* Fri Oct 11 2019 tv <tv> 3.3.1-2.mga8
+ Revision: 1451525
- rebuild for python-3.8
- drop python2 subpkg

* Wed Aug 28 2019 daviddavid <daviddavid> 3.3.1-1.mga8
+ Revision: 1434848
- new version: 3.3.1
- remove merged upstream patches
+ wally <wally>
- drop devel requires from doc pkg

* Fri Oct 12 2018 tv <tv> 3.2.1-5.mga7
+ Revision: 1319891
- fix deps (doc cannot be noarch if it depends on devel pkg)

* Thu Oct 04 2018 daviddavid <daviddavid> 3.2.1-4.mga7
+ Revision: 1317659
- obsolete another old remaining mpich2 pkg

* Sun Sep 23 2018 umeabot <umeabot> 3.2.1-3.mga7
+ Revision: 1299624
- Mageia 7 Mass Rebuild

* Tue Jun 12 2018 wally <wally> 3.2.1-2.mga7
+ Revision: 1236572
- make lib pkg to conflict with old lib pkg from openmpi

* Sun Jun 10 2018 wally <wally> 3.2.1-1.mga7
+ Revision: 1236178
- add upstream patch to build with hwloc >= 2.0
- fork as mpich
- new version 3.2.1
- sync .spec partially with Fedora

* Wed Sep 13 2017 tv <tv> 1.2.1-10.mga7
+ Revision: 1153502
- fix group
- add python subpkgs

* Thu Feb 04 2016 umeabot <umeabot> 1.2.1-8.mga6
+ Revision: 935213
- Mageia 6 Mass Rebuild
+ akien <akien>
- New upstream URL

* Wed Oct 15 2014 umeabot <umeabot> 1.2.1-7.mga5
+ Revision: 751023
- Second Mageia 5 Mass Rebuild

* Sat Sep 27 2014 tv <tv> 1.2.1-6.mga5
+ Revision: 727361
- rebuild for missing pythoneggs deps

* Tue Sep 16 2014 umeabot <umeabot> 1.2.1-5.mga5
+ Revision: 682651
- Mageia 5 Mass Rebuild

* Fri Oct 18 2013 umeabot <umeabot> 1.2.1-4.mga4
+ Revision: 518050
- Mageia 4 Mass Rebuild

* Sat Jan 12 2013 umeabot <umeabot> 1.2.1-3.mga3
+ Revision: 360436
- Mass Rebuild - https://wiki.mageia.org/en/Feature:Mageia3MassRebuild

* Sun Apr 08 2012 fwang <fwang> 1.2.1-2.mga2
+ Revision: 229659
- rebuild for new gcc
+ grenoya <grenoya>
- clean spec
  > - remove old patch #0
  > - fix man1 pages extention
- imported package mpich2

